// Set the dimensions and margins of the graph
const margin = { top: 20, right: 30, bottom: 70, left: 60 },
  width = 960 - margin.left - margin.right,
  height = 500 - margin.top - margin.bottom;

// Select the div with id 'my_dataviz' and remove any previous SVG elements before appending a new SVG
const svgContainer = d3.select('#my_dataviz');
svgContainer.selectAll('svg').remove();

// Append an SVG element to the container with defined width and height, and create a group element within it
const svg = svgContainer
  .append('svg')
  .attr('width', width + margin.left + margin.right)
  .attr('height', height + margin.top + margin.bottom)
  .append('g')
  .attr(
    'transform',
    `translate(${margin.left},${margin.top})`,// Translate the group element to accommodate margins
  );

// Read the data from a CSV file hosted online
d3.csv(
  'https://raw.githubusercontent.com/VedasriTulabandula/diabetes/main/diabetes.csv',
)
  .then(function (data) {
    // Group the data by Age and calculate mean Outcome for each age
    var sumstat = d3.rollup(
      data,
      (v) => d3.mean(v, (d) => d.Outcome),// Calculate the mean Outcome for each age group
      (d) => d.Age,// Group data by Age
    );
    // Convert the grouped data into an array of objects and sort it by age
    var meanOutcomeByAge = Array.from(
      sumstat,
      ([key, value]) => ({ Age: key, Outcome: value }),
    ).sort((a, b) => d3.ascending(+a.Age, +b.Age));

    // Add X axis
    const x = d3
      .scaleLinear()
      .domain(
        d3.extent(meanOutcomeByAge, function (d) {
          return +d.Age;// Set the domain of X axis to the range of ages in the data
        }),
      )
      .range([0, width]);// Set the range of X axis within the width of the graph
    svg
      .append('g')
      .attr('transform', `translate(0, ${height})`)// Translate X axis to the bottom of the graph
      .call(d3.axisBottom(x))// Append X axis to the SVG
      .selectAll('text')
      .style('text-anchor', 'end')
      .attr('dx', '-.8em')
      .attr('dy', '.15em')
      .attr('transform', 'rotate(-65)');// Rotate X axis labels for better readability

    // Add Y axis
    const y = d3
      .scaleLinear()
      .domain([0, 1])// Set the domain of Y axis to range from 0 to 1 (Outcome values)
      .range([height, 0]);// Set the range of Y axis within the height of the graph
    svg.append('g').call(d3.axisLeft(y));// Append Y axis to the SVG

    // Add the line representing the trend of Outcome by Age
    svg
      .append('path')
      .datum(meanOutcomeByAge)// Bind data to the path element
      .attr('fill', 'none')// Set fill color to none

      .attr('stroke', 'steelblue')
      .attr('stroke-width', 2)// Set stroke width
      .attr(
        'd',
        d3
          .line()
          .x(function (d) {
            return x(+d.Age);// Set X coordinate based on Age
          })
          .y(function (d) {
            return y(d.Outcome);// Set Y coordinate based on Outcome
          }),
      );// Generate the path for the line using d3.line

    // Text label for the x axis
    svg
      .append('text')
      .attr(
        'transform',
        'translate(' +
          width / 2 +
          ' ,' +
          (height + margin.top + 50) +
          ')',
      )// Position the label at the bottom center of the graph
      .style('text-anchor', 'middle')
      .text('Age');

    // Text label for the y axis
    svg
      .append('text')
      .attr('transform', 'rotate(-90)')
      .attr('y', 0 - margin.left)
      .attr('x', 0 - height / 2)
      .attr('dy', '1em')
      .style('text-anchor', 'middle')
      .text('Mean Outcome');

    // Title
    svg
      .append('text')
      .attr('x', width / 2)
      .attr('y', 0 - margin.top / 2)
      .attr('text-anchor', 'middle')
      .style('font-size', '24px')
      .style('text-decoration', 'underline')
      .text('Diabetes Outcome by Age');
  })
  .catch(function (error) {
    console.log(error);// Log any errors that occur during data loading
  });
